package com.motse.fengshui_compass.common;

import android.os.Handler;
import android.os.HandlerThread;
import android.serialport.SerialPort;


import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import io.reactivex.rxjava3.core.Scheduler;
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers;

public class SerialPortManager {
    private static final String TAG = "SerialPortManager";
    private SerialReadThread mReadThread;
    private OutputStream mOutputStream;
    private HandlerThread mWriteThread;
    private Scheduler mSendScheduler;
//    private Handler handler;

    private static class InstanceHolder {
        public static SerialPortManager sManager = new SerialPortManager();
    }

    public static SerialPortManager instance() {
        return InstanceHolder.sManager;
    }

    private SerialPort mSerialPort;


    private SerialPortManager() {

    }

    /**
     * 打开串口
     *
     * @param device
     * @return
     */
    public SerialPort open(Device device, Handler dataHandle) {
//        this.handler = dataHandle;
        return open(device.getPath(), device.getBaudrate(), dataHandle);
    }

    /**
     * 打开串口
     *
     * @param devicePath
     * @param baudrateString
     * @return
     */
    public SerialPort open(String devicePath, String baudrateString,Handler handler) {
        if (mSerialPort != null) {
            close();
        }
        try {
            File device = new File(devicePath);
            int baurate = Integer.parseInt(baudrateString);
            mSerialPort = new SerialPort(device, baurate);
//            if (handler != null) {
            mReadThread = new SerialReadThread(mSerialPort.getInputStream(), handler);
//            } else {
//                mReadThread = new SerialReadThread(mSerialPort.getInputStream());
//            }
            mReadThread.start();

            mOutputStream = mSerialPort.getOutputStream();

            mWriteThread = new HandlerThread("write-thread");
            mWriteThread.start();

            mSendScheduler = AndroidSchedulers.from(mWriteThread.getLooper());

            return mSerialPort;
        } catch (Throwable tr) {
//            LogPlus.e(TAG, "打开串口失败", tr);
            System.out.println("打开串口失败");
            close();
            return null;
        }
    }

    /**
     * 关闭串口
     */
    public void close() {
        if (mReadThread != null) {
            mReadThread.close();
        }
        if (mOutputStream != null) {
            try {
                mOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (mWriteThread != null) {
            mWriteThread.quit();
        }

        if (mSerialPort != null) {
            mSerialPort.close();
            mSerialPort = null;
        }
    }

    /**
     * 发送数据
     *
     * @param datas
     * @return
     */
    public void sendData(byte[] datas) throws Exception {
        mOutputStream.write(datas);
    }
}
